@@ -55,6 +55,8 @@
 
 #define PAM_SM_AUTH
 #define PAM_SM_ACCOUNT
+#define PAM_SM_SESSION
+#define PAM_SM_PASSWORD
 
 #include <security/_pam_macros.h>
 #include <security/pam_modules.h>
@@ -535,7 +537,7 @@ from_match (pam_handle_t *pamh UNUSED, char *tok, struct login_info *item)
       struct addrinfo hint;
 
       memset (&hint, '\0', sizeof (hint));
-      hint.ai_flags = AI_ADDRCONFIG | AI_CANONNAME;
+      hint.ai_flags = AI_CANONNAME;
       hint.ai_family = AF_INET;
 
       if (getaddrinfo (string, NULL, &hint, &res) != 0)
@@ -561,10 +563,10 @@ from_match (pam_handle_t *pamh UNUSED, char *tok, struct login_info *item)
 		      freeaddrinfo (res);
 		      return YES;
 		    }
-		  runp = runp->ai_next;
 		}
-	      freeaddrinfo (res);
+	      runp = runp->ai_next;
 	    }
+	  freeaddrinfo (res);
 	}
     } else  if (isipaddr(string, NULL, NULL) == YES) {
       /* Assume network/netmask with a IP of a host.  */
@@ -576,7 +578,7 @@ from_match (pam_handle_t *pamh UNUSED, char *tok, struct login_info *item)
       struct addrinfo hint;
 
       memset (&hint, '\0', sizeof (hint));
-      hint.ai_flags = AI_ADDRCONFIG | AI_CANONNAME;
+      hint.ai_flags = AI_CANONNAME;
       hint.ai_family = AF_UNSPEC;
 
       if (getaddrinfo (string, NULL, &hint, &res) != 0)
@@ -811,6 +813,27 @@ pam_sm_acct_mgmt (pam_handle_t *pamh, int flags,
   return pam_sm_authenticate (pamh, flags, argc, argv);
 }
 
+PAM_EXTERN int
+pam_sm_open_session(pam_handle_t *pamh, int flags,
+		    int argc, const char **argv)
+{
+  return pam_sm_authenticate(pamh, flags, argc, argv);
+}
+
+PAM_EXTERN int
+pam_sm_close_session(pam_handle_t *pamh, int flags,
+		     int argc, const char **argv)
+{
+  return pam_sm_authenticate(pamh, flags, argc, argv);
+}
+
+PAM_EXTERN int
+pam_sm_chauthtok(pam_handle_t *pamh, int flags,
+		 int argc, const char **argv)
+{
+  return pam_sm_authenticate(pamh, flags, argc, argv);
+}
+
 /* end of module definition */
 
 #ifdef PAM_STATIC
@@ -822,8 +845,8 @@ struct pam_module _pam_access_modstruct = {
     pam_sm_authenticate,
     pam_sm_setcred,
     pam_sm_acct_mgmt,
-    NULL,
-    NULL,
-    NULL
+    pam_sm_open_session,
+    pam_sm_close_session,
+    pam_sm_chauthtok
 };
 #endif
