@@ -33,8 +33,6 @@
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-/* "$Id: pam_xauth.c,v 1.12 2005/11/23 16:42:40 kukuk Exp $" */
-
 #include "config.h"
 #include <sys/types.h>
 #include <sys/fsuid.h>
@@ -52,6 +50,8 @@
 #include <syslog.h>
 #include <unistd.h>
 
+#define PAM_SM_SESSION
+
 #include <security/pam_modules.h>
 #include <security/_pam_macros.h>
 #include <security/pam_modutil.h>
@@ -665,3 +665,16 @@ pam_sm_close_session (pam_handle_t *pamh, int flags UNUSED,
 	}
 	return PAM_SUCCESS;
 }
+
+/* static module data */
+#ifdef PAM_STATIC
+struct pam_module _pam_xauth_modstruct = {
+  "pam_xauth",
+  NULL,
+  NULL,
+  NULL,
+  pam_sm_open_session,
+  pam_sm_close_session,
+  NULL
+};
+#endif
